/*
 *  This file is part of uIRC. (https://git.redxen.eu/caskd/uIRC)
 *  Copyright (c) 2019-2021 Alex-David Denes
 *
 *  uIRC is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  uIRC is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with uIRC.  If not, see <https://www.gnu.org/licenses/>.
 */

/*! \file */

#include "mode.h" // IRC_Modes
#include "type.h" // IRC_*

#include <stddef.h> // ptrdiff_t

#ifndef UIRC_GUARD_PUBLIC_MEMORY
#define UIRC_GUARD_PUBLIC_MEMORY

void uirc_struct_free(void* obj, IRC_Struct_Type type);

#ifdef UIRC_FEATURE_IRCV3
IRC_Tag*        uirc_struct_assm_tag(bool clientbound, const char* key, const char* value);
IRC_Capability* uirc_struct_assm_capability(bool active, const char* name);
#endif /* UIRC_FEATURE_IRCV3 */
IRC_User*    uirc_struct_assm_user(bool service, IRC_Modes* modes, const char* nick, const char* user, const char* real, const char* host);
IRC_Message* uirc_struct_assm_message(const char* cmd, bool trailing, unsigned int argc, ...);
IRC_Buffer*  uirc_struct_assm_buffer(IRC_Buffer_Type type, IRC_Modes* modes, bool subscribed, const char* name, const char* key, const char* topic);
IRC_Network* uirc_struct_assm_network(const char* addr, const char* svc, const char* pass);

void*     uirc_list_append(llist_t* anchor, void* content);
ptrdiff_t uirc_list_free(llist_t* list, IRC_Struct_Type type);

#endif /* UIRC_GUARD_PUBLIC_MEMORY */

